@@ -298,6 +298,10 @@ def do_build_base(args):
   )
 
 def do_build(args):
+  if not args.snapshot_on_failure:
+    args.snapshot_on_failure = []
+  for subargs in args.snapshot_on_failure:
+    subargs[1] = int(subargs[1])
   nonce = binascii.b2a_hex(os.urandom(15)).decode('utf8')
   archive = '%s-%s' % (datetime.datetime.now().strftime('%Y%m%d'), nonce)
   print('Sending build context...')
@@ -467,7 +471,22 @@ def main():
 
   build_parser = subparsers.add_parser('build', help='Build a Visorfile')
   build_parser.add_argument('--keep-running', action='store_true')
-  build_parser.add_argument('--snapshot-on-failure', action='store', nargs='?', const=True, default=None)
+  build_parser.add_argument(
+    '--snapshot-on-failure',
+    action='append',
+    nargs=3,
+    metavar=('failure_string', 'max_snapshot_rate_in_minutes', 'note_string'),
+    help='''
+      Takes 3 arguments.
+      The first argument is a failure string to match for. Snapshots will only
+      be taken after a failure if the failing command contains this string.
+      The second argument is a number representing how many minutes to wait
+      since the last time a snapshot of the same failure string was taken.
+      Set to 0 if you want to always take snapshots of the given failure string.
+      The third argument is a note which you can use to store arbitrary metadata
+      about the snapshot.
+    '''
+  )
   build_parser.add_argument('--build-env', dest='buildenvs', action='append', default=[])
   build_parser.add_argument('visorfile', help='The Visorfile to build')
   build_parser.set_defaults(func=do_build)
